Package: fgga
Type: Package
Title: Hierarchical ensemble method based on factor graph
Version: 1.18.0
Date: 2023-11-27
Authors@R: c(person("Flavio", "Spetale", email = "spetale@cifasis-conicet.gov.ar", role = c("aut", "cre")))
biocViews: Software, StatisticalMethod, Classification, Network,
        NetworkInference, SupportVectorMachine, GraphAndNetwork, GO
Description: Package that implements the FGGA algorithm. This package provides a hierarchical ensemble method based ob factor graphs for the consistent cross-ontology annotation of protein coding genes. FGGA embodies elements of predicate logic, communication theory, supervised learning and inference in graphical models.
Depends: R (>= 4.3), RBGL
Imports: graph, stats, e1071, methods, gRbase, jsonlite, BiocFileCache,
        curl, igraph
Suggests: knitr, rmarkdown, GOstats, GO.db, BiocGenerics, pROC, RUnit,
        BiocStyle
License: GPL-3
URL: https://github.com/fspetale/fgga
Encoding: UTF-8
LazyData: false
VignetteBuilder: knitr
NeedsCompilation: no
git_url: https://git.bioconductor.org/packages/fgga
git_branch: RELEASE_3_22
git_last_commit: 2621333
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-29
Packaged: 2025-10-30 03:57:49 UTC; biocbuild
Author: Flavio Spetale [aut, cre]
Maintainer: Flavio Spetale <spetale@cifasis-conicet.gov.ar>

export(fgga)
export(fgga2bipartite)
export(svmTrain)
export(svmOnto)
export(msgFGGA)
export(tableTPG)
export(preCoreFG)
export(varianceOnto)
export(createFolds)
export(maxDistancegraphOnto)
export(fHierarchicalMeasures)
export(fMeasures)
export(fMeasuresByLevel)

import("RBGL")
import("stats")
import("BiocFileCache")
importFrom("jsonlite", fromJSON)
importFrom("curl", has_internet)
importFrom("stats", "na.omit", "predict", "setNames")
importFrom("methods", "as", "is")
importFrom("graph", subGraph)
importFrom("gRbase", coerceGraph)
importFrom("gRbase", graph_as)
importFrom("igraph", graph_from_graphnel)
importFrom("gRbase", ancestralSet)
importFrom("gRbase", parents)
importFrom("gRbase", children)
importFrom("e1071", "svm")
importFrom("graph", leaves)
importFrom("graph", nodes)
importFrom("graph", graphNEL)

----------
# fgga

# R package

# FGGA: Factor Graph Gene ontology Annotation

FGGA is a graph-based machine learning approach for the automated and consistent GO annotation of protein coding genes. The input is a set of GO-term annotated protein coding genes previously characterized in terms of a fixed number of user-defined features, including the presence/absence of PFAM domains, physical-chemical properties, presence of signal peptides, among others. The set of GO-terms defines the output GO subgraph. A hierarchical ensemble (SVMs) machine learning model is generated. This model can be used to predict the GO subgraph annotations of uncharacterized protein coding genes. Individual GO-term annotations are accompanied by maximum a posteriori probability estimates issued by the native message passing algorithm of factor graphs.


# INSTALLATION

The fgga R source package can be directly downloaded from [Bioconductor repository](https://bioconductor.org/) or [GitHub repository](https://github.com/fspetale/fgga). This R package contains a experimental dataset as example, one pre-run R object and all functions needed to run FGGA.

\## From Bioconductor repository

if (!requireNamespace("BiocManager", quietly = TRUE)) {
      install.packages("BiocManager")}
      
BiocManager::install("fgga")

\## Or from GitHub repository using devtools

BiocManager::install("devtools")

devtools::install_github("fspetale/fgga")

# REFERENCES

1: Spetale F.E., Tapia E., Krsticevic F., Roda F. and Bulacio P. “A Factor Graph Approach to Automated GO Annotation”. PLoS ONE 11(1): e0146986, 2016.

2: Spetale Flavio E., Arce D., Krsticevic F., Bulacio P. and Tapia E. “Consistent prediction of GO protein localization”. Scientific Report 7787(8), 2018

createFolds <- function(target, k_fold = 10) {
    target <- as.factor(target)
    if (k_fold < length(target)) {
        target <- factor(as.character(target))
        numInClass <- table(target)
        foldVector <- vector(mode = "integer", length(target))
        for (i in seq_len(length(numInClass))) {
            min_reps <- numInClass[i] %/% k_fold
            if (min_reps > 0) {
                spares <- numInClass[i] %% k_fold
                seqVector <- rep(seq_len(k_fold), min_reps)
                if (spares > 0) {
                    seqVector <- c(seqVector, sample((k_fold), spares))
                }
                foldVector[which(target == names(numInClass)[i])] <-
                    sample(seqVector)
            }
            else {
                foldVector[which(target == names(numInClass)[i])] <-
                    sample(seq_len(k_fold), size = numInClass[i])
            }
        }
    }
    outfoldVector <- split(seq(along = target), foldVector)
    names(outfoldVector) <- paste("Fold", gsub(" ", "0", format(
        seq(along = outfoldVector))), sep = "")
    outfoldVector
}

.productMsg <- function(valuesTPG, matrixFunction, matrixNode, index1, index2) {
    valueProduct <- 1
    for (z in seq_len(length(valuesTPG) - 1)) {
        valuesMsg <- matrixNode[matrixFunction[index1, index2][[1]]$neighbor[z],
                    index1][[1]][[valuesTPG[z]]]
        valueProduct <- valuesMsg * valueProduct
    }
    return(valueProduct)
}

msgFGGA <- function(matrixFGGA, obsValueOntoTerms, graphOnto, tmax = 200,
                    epsilon = 0.001) {
    if (!is(matrixFGGA, 'fgga')){
        message("matrixFGGA is not class fgga")
        return()
    } else {
        matrixFGGA <- as(matrixFGGA, "matrix")
    }
    nFactors <- dim(matrixFGGA)[2]
    nNodes <- dim(matrixFGGA)[1] / 2
    matrixOnto <- as(graphOnto, "matrix")
    matrixNodeToFunction <- array(data = list(), dim = c(nNodes, nFactors))
    matrixFunctionToNode <- array(data = list(), dim = c(nFactors, nNodes))
    rownames(matrixNodeToFunction) <- rownames(matrixFGGA)[seq_len(nNodes)]
    colnames(matrixFunctionToNode) <- rownames(matrixFGGA)[seq_len(nNodes)]
    colnames(matrixNodeToFunction) <- colnames(matrixFGGA)
    rownames(matrixFunctionToNode) <- colnames(matrixFGGA)

    # Initialization
    for (i in seq_len(dim(matrixFunctionToNode)[2])) {
        matrixFunctionToNode[nNodes - 1 + i, i][[1]] <- list(
            zeroValues = 1 - obsValueOntoTerms[i],
            oneValues = obsValueOntoTerms[i], valueOnto = 1)
        if (i < nNodes) {
            nodeLinks <- which(matrixFGGA[, i] == 1)
            submatrixOnto <- matrixOnto[nodeLinks, nodeLinks]
            leafOnto <- names(which(apply(submatrixOnto, MARGIN = 1, sum) == 0))
            parentsOnto <- setdiff(colnames(submatrixOnto), leafOnto)
            functionLink <- t(apply(t(names(nodeLinks)), MARGIN=2,
                FUN = function(x, y, w) {(if (x == w) setdiff(y, x) else c(
                w, setdiff(y, c(x, w))))}, w = leafOnto, y = names(nodeLinks)))
            for (k in seq_len(length(nodeLinks))) {
                    if (length(nodeLinks) > 2) {
                        matrixFunctionToNode[i, nodeLinks[k]][[1]] <-
                            list(zeroValues = 1, oneValues = 1,
                            neighbor = functionLink[k, ], leafOnto = leafOnto,
                            parentsOnto = parentsOnto,
                            TPG = length(nodeLinks) - 1)
                    } else {
                        matrixFunctionToNode[i, nodeLinks[k]][[1]] <- list(
                            zeroValues = 1,
                            oneValues = 1, neighbor = functionLink[1, k],
                            leafOnto = leafOnto, parentsOnto = parentsOnto,
                            TPG = length(nodeLinks) - 1)
                    }
            }
        }
        nodeLinks <- which(matrixFGGA[i, ] == 1)
        functionLink <- t(apply(t(names(nodeLinks)), MARGIN=2,
            FUN = function(x, y) {(setdiff(y, x))}, y = names(nodeLinks)))
        for (k in seq_len(length(nodeLinks))) {
                if (length(nodeLinks) > 2) {
                    matrixNodeToFunction[i, nodeLinks[k]][[1]] <- list(
                    zeroValues = 1, oneValues = 1, neighbor = functionLink[k, ])
                } else {
                    matrixNodeToFunction[i, nodeLinks[k]][[1]] <- list(
                    zeroValues = 1, oneValues = 1, neighbor = functionLink[1, k]
                    )
                }
        }
    }
    rm(submatrixOnto, leafOnto, parentsOnto)
    for (t in seq_len(tmax)) {
        # Msg node variable to function
        for (i in seq_len(nNodes)) {
            activeFunction <- which(matrixFGGA[i, ] == 1)
            for (j in seq_len(length(activeFunction))) {
                valuesMsg <- apply(t(activeFunction[-j]), MARGIN = 2,
                    FUN = function(x, y, z) (y[x, z][[1]]$oneValues),
                    y = matrixFunctionToNode, z = i)
                matrixNodeToFunction[i, activeFunction[j]][[1]]$oneValues <-
                    prod(valuesMsg)
                valuesMsg <- apply(t(activeFunction[-j]), MARGIN = 2,
                    FUN = function(x, y, z) (y[x, z][[1]]$zeroValues),
                    y = matrixFunctionToNode, z = i)
                matrixNodeToFunction[i, activeFunction[j]][[1]]$zeroValues <-
                    prod(valuesMsg)
                gammaMsg <- 1 / (matrixNodeToFunction[i,
                activeFunction[j]][[1]]$oneValues + matrixNodeToFunction[i,
                        activeFunction[j]][[1]]$zeroValues)
                matrixNodeToFunction[i, activeFunction[j]][[1]]$oneValues <-
                    matrixNodeToFunction[i,
                    activeFunction[j]][[1]]$oneValues * gammaMsg
                matrixNodeToFunction[i, activeFunction[j]][[1]]$zeroValues <-
                    matrixNodeToFunction[i,
                    activeFunction[j]][[1]]$zeroValues * gammaMsg
            }
        }
        # Msg function to node variable
        for (i in seq_len(nNodes - 1)) {
            activeFunction <- which(matrixFGGA[, i] == 1)
            for (j in seq_len(length(activeFunction))) {
                functionsTPG <- tableTPG(matrixFunctionToNode[i,
                                        activeFunction[j]][[1]]$TPG+1)
                if (matrixFunctionToNode[i, activeFunction[j]][[1]]$leafOnto ==
                    colnames(matrixFunctionToNode)[activeFunction[j]]) {
                    for (k in seq(1, 2)) {
                        indexMsg <- which(functionsTPG[, 1] == k)
                        subMatrixTPG <- functionsTPG[indexMsg, -1]
                        if (is.matrix(subMatrixTPG)) {
                            valuesMsg <- apply(subMatrixTPG,
                                MARGIN = 1, FUN = .productMsg,
                                index1 = i, index2 = activeFunction[j],
                                matrixFunction = matrixFunctionToNode,
                                matrixNode = matrixNodeToFunction)
                        } else {
                                valuesMsg <- apply(as.matrix(t(subMatrixTPG)),
                                    MARGIN = 1, FUN = .productMsg, index1 = i,
                                    index2 = activeFunction[j],
                                    matrixFunction = matrixFunctionToNode,
                                    matrixNode = matrixNodeToFunction)
                            }
                        if (k == 1) {
                            matrixFunctionToNode[i,
                                activeFunction[j]][[1]]$zeroValues <-
                                sum(valuesMsg)
                        } else {
                            matrixFunctionToNode[i,
                                activeFunction[j]][[1]]$oneValues <-
                                sum(valuesMsg)
                        }
                    }
                } else {
                    for (k in seq(1, 2)) {
                        indexMsg <- which(functionsTPG[, 2] == k)
                        subMatrixTPG <- functionsTPG[indexMsg, -2]
                        if (is.matrix(subMatrixTPG)) {
                            valuesMsg <- apply(subMatrixTPG, MARGIN = 1,
                                    FUN = .productMsg, index1 = i,
                                    index2 = activeFunction[j],
                                    matrixFunction = matrixFunctionToNode,
                                    matrixNode = matrixNodeToFunction)
                        } else {
                                valuesMsg <- apply(as.matrix(t(subMatrixTPG)),
                                MARGIN = 1, FUN = .productMsg, index1 = i,
                                index2 = activeFunction[j],
                                matrixFunction = matrixFunctionToNode,
                                matrixNode = matrixNodeToFunction)
                        }
                        if (k == 1) {
                            matrixFunctionToNode[i,
                            activeFunction[j]][[1]]$zeroValues <- sum(valuesMsg)
                        } else {
                            matrixFunctionToNode[i,
                            activeFunction[j]][[1]]$oneValues <- sum(valuesMsg)}
                    }
                }
                gammaMsg <- 1 / (matrixFunctionToNode[i,
                    activeFunction[j]][[1]]$oneValues +
                    matrixFunctionToNode[i, activeFunction[j]][[1]]$zeroValues)
                matrixFunctionToNode[i, activeFunction[j]][[1]]$oneValues <-
                    matrixFunctionToNode[i, activeFunction[j]][[1]]$oneValues *
                        gammaMsg
                matrixFunctionToNode[i, activeFunction[j]][[1]]$zeroValues <-
                    matrixFunctionToNode[i, activeFunction[j]][[1]]$zeroValues *
                        gammaMsg
            }
        }
        ValuesOnto <- lapply(diag(matrixNodeToFunction[,
            seq(nNodes, nFactors)]), FUN = function(x) (x[c(1, 2)]))
        oneValuesOnto <- vapply(seq(1, nNodes), FUN = function(x, y, z) {
            (y[[x]]$oneValues * z[x])}, y = ValuesOnto, z = obsValueOntoTerms,
            FUN.VALUE = numeric(1))
        zeroValuesOnto <- vapply(seq(1, nNodes), FUN = function(x, y, z) {
            (y[[x]]$zeroValues * (1 - z[x]))}, y = ValuesOnto,
            z = obsValueOntoTerms, FUN.VALUE = numeric(1))
        oneValuesOnto <- vapply(seq(1, nNodes), FUN = function(x, y, z) {
            (y[x] / (y[x] + z[x]))}, y = oneValuesOnto, z = zeroValuesOnto,
            FUN.VALUE = numeric(1))
        ValuesOnto <- vapply(diag(matrixFunctionToNode[
            seq(nNodes, nFactors), ]), FUN = function(x) (x$valueOnto),
            FUN.VALUE = numeric(1))
        errorGO <- vapply(seq(1, nNodes), FUN = function(x, y, z) {
            (abs(y[x] - z[x]))}, y = oneValuesOnto, z = ValuesOnto,
            FUN.VALUE = numeric(1))
        if (max(errorGO) < epsilon) {
            names(oneValuesOnto) <- colnames(matrixFunctionToNode)
            return(oneValuesOnto)
        } else {
            for (i in seq_len(nNodes)) {
                matrixFunctionToNode[nNodes - 1 + i, i][[1]]$valueOnto <-
                    oneValuesOnto[i]}
        }
    }
    ValuesOnto <- array(0, nNodes)
    names(ValuesOnto) <- colnames(matrixFunctionToNode)
    message("Graph UNCONVERGED")
    return(ValuesOnto)
}

maxDistancegraphOnto <- function(graphOnto){
    terms <- nodes(graphOnto)
    matrixDistgraphOnto <- matrix(0,length(terms),2)
    matrixDistgraphOnto[1, 1] <- gRoot <- leaves(graphOnto, "in")
    matrixDistgraphOnto <- matrix("list", length(terms))
    termUsed <- array(0, length(terms))
    names(matrixDistgraphOnto) <- names(termUsed) <- terms
    gLeaves <- leaves(graphOnto, "out")
    pNodes <- terms
    while(length(pNodes)>0){
        pathParents <- parents(pNodes[1], graph_from_graphnel(graphOnto))
        if (length(pNodes) == length(terms)){
            pNodes <- setdiff(terms, gRoot)
            matrixDistgraphOnto[gRoot] <- gRoot
            termUsed[gRoot] <- 1
        }
        if (length(pathParents) > 0){
            allNodes <- intersect(pathParents, names(
                termUsed[which(termUsed > 0)]))
            if (length(allNodes) == length(pathParents)){
                matrixDistgraphOnto[pNodes[1]] <- as.data.frame(c(pNodes[1],
                as.character(unlist(matrixDistgraphOnto[pathParents[
                which.max(termUsed[pathParents])]]))))
            termUsed[pNodes[1]] <- termUsed[pathParents[
                which.max(termUsed[pathParents])]] + length(pNodes[1])
            pNodes <- pNodes[-1]
            } else {
            nodesUnused <- setdiff(pathParents,names(termUsed[
                which(termUsed > 0)]))
            pLastNode <- which(pNodes == nodesUnused[length(nodesUnused)])
            if (pLastNode == length(pNodes)){
                pNodes <- c(pNodes[seq(2, pLastNode)], pNodes[1])
            } else pNodes<-c(pNodes[seq(2, pLastNode)], pNodes[1],
                            pNodes[seq((pLastNode + 1), length(pNodes))])
            }
        }
    }
    return(termUsed=termUsed-1)
}

svmTrain <- function(nodeGraph, tableOntoTerms, dxCharacterized, graphOnto,
                        kernelSVM = "radial") {
    message("Term: ", nodeGraph)
    tableOntoTerms <- tableOntoTerms[order(rownames(tableOntoTerms)), ]
    dxCharacterized <- dxCharacterized[order(rownames(dxCharacterized)), ]
    modelSvm_reduce <- c()
    zeroValue <- array(0, 0.1 * dim(dxCharacterized)[2])
    #idAncestor <- ancestors(nodeGraph, graphOnto)
    mygraphGO <- graph_from_graphnel(graphOnto)
    idAncestor <- ancestralSet(nodeGraph, mygraphGO)
    rootNodes <- c("GO:0008150", "GO:0003674", "GO:0005575", "PO:0025131",
                    "PO:0009012", "ZFA:0100000", "HP:0000001", "Ontology:FES")
    rootNodes <- intersect(idAncestor, rootNodes)
    idAncestor <- setdiff(idAncestor, rootNodes)
    indices <- which(tableOntoTerms[, nodeGraph] == 1)
    positiveGeneNames <- rownames(tableOntoTerms)[indices]
    notNegativeGeneNames <- positiveGeneNames
    if (length(idAncestor) > 0) {
        for (i in seq_len(length(idAncestor))) {
            buffer <- rownames(tableOntoTerms)[which(tableOntoTerms[,
                        idAncestor[i]] == 1)]
            notNegativeGeneNames <- union(notNegativeGeneNames, buffer)
        }
    }

    negativeGeneNames <- setdiff(rownames(tableOntoTerms), notNegativeGeneNames)
    np <- length(positiveGeneNames)
    nn <- length(negativeGeneNames)
    if ((np != 0) && (nn != 0)) {
        if (nn > 2 * np || np > 2 * nn) {
            if (nn > 2 * np) {
                nn <- 2 * np
                id.negative <- sample(seq_len(length(negativeGeneNames)), nn)
                exprsValues <- dxCharacterized[c(
                    positiveGeneNames, negativeGeneNames[id.negative]), ]
            }
            else {
                np <- 2 * nn
                id.positive <- sample(seq_len(length(positiveGeneNames)), np)
                exprsValues <- dxCharacterized[c(
                    positiveGeneNames[id.positive], negativeGeneNames), ]
            }
        }
        else {
            exprsValues <- dxCharacterized[c(
                positiveGeneNames, negativeGeneNames), ]
        }
        exprsValues[is.na(exprsValues)] <- 0
        terms <- as.factor(c(rep(1, np), rep(2, nn)))
        zeroValue <- apply(exprsValues, MARGIN = 2, mean)
        zeroValue <- which(zeroValue == 0)
        if (length(zeroValue) != 0) exprsValues <- exprsValues[, -zeroValue]
        modelSvm <- svm(exprsValues, terms, method = "C-classification",
                        cost = 500, kernel = kernelSVM, na.action = na.omit)
        modelSvm_reduce <- modelSvm[-c(28, 29)]
        class(modelSvm_reduce) <- class(modelSvm)
        modelSvm_reduce$zeroValue <- zeroValue
    }
    modelSvm_reduce$ontoTerm <- nodeGraph
    return(modelSvm_reduce)
}

svmOnto <- function(svmMoldel, dxCharacterized, rootNode, varianceSVM) {
    dxTestSVM <- lapply(svmMoldel, FUN = .svmTest, dxTest = dxCharacterized,
        rootOnto = rootNode, variance = varianceSVM)
    matrixTest <- vapply(dxTestSVM, cbind,
                        FUN.VALUE = double(dim(dxTestSVM[[1]])[1]))
    colnames(matrixTest) <- vapply(dxTestSVM, colnames,
                                    FUN.VALUE = character(1))
    rownames(matrixTest) <- vapply(dxTestSVM[1], rownames,
                                FUN.VALUE = character(dim(dxTestSVM[[1]])[1]))
    return(matrixTest)
}

.svmTest <- function(svmMoldel, dxTest, rootOnto, variance) {
    dxTest <- as.data.frame(dxTest)
    matrixScore <- matrix(0, dim(dxTest)[1], 1)
    rownames(matrixScore) <- rownames(dxTest)
    nodeOnto <- svmMoldel$ontoTerm
    colnames(matrixScore) <- nodeOnto
    if (nodeOnto != rootOnto) {
        zeroValues <- svmMoldel$zeroValue
        if (length(zeroValues) != 0) dxTest <- dxTest[, -zeroValues]
        predOnto <- predict(svmMoldel, dxTest, decision.values = TRUE)
        matrixScore[, 1] <- 1 / (1 + exp(-2 * attr(predOnto, "decision.values")/
            variance[nodeOnto]))
    } else {
        matrixScore[, 1] <- 0.9999
    }
    matrixScore[which(matrixScore < 1.0e-06)] <- 1.0e-06
    matrixScore[which(matrixScore > 0.9999999)] <- 0.9999
    return(matrixScore)
}

library(fgga)
test_graph <- function(){
    V <- sapply(seq_len(9), FUN = function(x) {
        if (x == 6) "Ontology:FES" else paste0("GO:000", x, sep = "")})
    g <- matrix(0, 9, 9)
    colnames(g) <- rownames(g) <- V
    g[1, 9] <- 1
    g[2, 9] <- 1
    g[3, c(2, 4, 7, 8)] <- 1
    g[4, 1] <- 1
    g[5, 1] <- 1
    g[6, 3] <- 1
    g[7, 5] <- 1
    g[8, 5] <- 1
    g <- as(g, "graphNEL")
    return (g)
}

test_svm <- function() {
    dx1 <- matrix(runif(360, 0, 1), 40, 9)
    dx3 <- matrix(runif(18, 0, 1), 2, 9)
    dx2 <- matrix(rbinom(360, 1, 0.3), 40, 9)
    dx2[, 6] <- rep(1, 40)
    colnames(dx1) <- colnames(dx2) <- colnames(dx3) <- V <- sapply(seq_len(9),
        FUN = function(x)
            { if (x == 6) "Ontology:FES" else paste0("GO:000", x, sep = "")})
    rownames(dx1) <- rownames(dx2) <-
        sapply(seq_len(40), FUN = function(x) paste0("ID", x, sep = ""))
    rownames(dx3) <- sapply(seq_len(2),
                            FUN = function(x) paste0("IDT", x, sep = ""))

    g <- test_graph()

    svmM <- lapply(V, FUN = svmTrain, tableOntoTerms = dx2, dxCharacterized = dx1,
                    graphOnto = g, kernelSVM = "radial")

    root <- leaves(g, "in")

    vx <- array(runif(9, 0, 1), 9)
    names(vx) <- colnames(dx1)
    classSVM <- vapply(svmM, class, FUN.VALUE = character(1))
    sinSVM <- which(classSVM=="list")
    if (length(sinSVM)>1){
        sinSVM <- setdiff(sinSVM, 6)
        svmM <- svmM[-sinSVM]
    }

    mT <- svmOnto(svmM, dx3, root, vx)

    checkTrue(is.numeric(mT))
    checkEquals(mT[1, 6], 0.9999)
}

fgga2bipartite <- function(graphOnto) {
    matrixOnto <- as(graphOnto, "matrix")
    matrixOnto <- matrixOnto[, order(colnames(matrixOnto))]
    matrixOnto <- matrixOnto[order(rownames(matrixOnto)), ]
    nOnto <- dim(matrixOnto)[1]
    matrixFGGA <- matrix(0, nOnto * 2, nOnto * 2 - 1)
    rownames(matrixFGGA) <- c(rownames(matrixOnto), vapply(rownames(matrixOnto),
        FUN = function(att) {(paste("Obs_", att, sep = "", collapse = ""))},
        FUN.VALUE = character(1)))
    k <- vapply(seq_len(dim(matrixOnto)[1] - 1), FUN = function(att) {paste(
        "f", att, sep = "", collapse = "")}, FUN.VALUE = character(1))
    colnames(matrixFGGA) <- c(k, vapply(seq_len(dim(matrixOnto)[1]),
        FUN = function(att) {paste("g", att, sep = "", collapse = "")},
        FUN.VALUE = character(1)))
    linksFunctions <- apply(matrixOnto, MARGIN = 2,
                            FUN = function(x) (which(x == 1)))
    k <- 1
    for (i in seq_len(nOnto)) {
        matrixFGGA[c(i, i + nOnto), i + nOnto - 1] <- 1
        if (length(linksFunctions[[i]]) != 0) {
            matrixFGGA[c(i, linksFunctions[[i]]), k] <- 1
            k <- k + 1
        }
    }
    class(matrixFGGA) <- "fgga"
    return(matrixFGGA)
}

library(fgga)
test_graph <- function(){
  V <- LETTERS[1:10]
  edL <- vector("list", length=length(V))
  names(edL) <- V
  edL[[1]] <- list(edges=c(2, 3, 7, 5))
  edL[[2]] <- list(edges=c(4, 6))
  edL[[3]] <- list(edges=c(6, 8))
  edL[[4]] <- list(edges=c(6, 9))
  edL[[5]] <- list(edges=c(8))
  edL[[6]] <- list(edges=c(8, 9, 10))
  edL[[7]] <- list(edges=c(8, 5))
  edL[[8]] <- list(edges=c())
  edL[[9]] <- list(edges=c())
  g <- graph::graphNEL(nodes=V, edgeL=edL, edgemode="directed")
  return(g)
}

test_msg <- function() {

  g <- test_graph()
  mF <- fgga2bipartite(g)
  mT <- matrix(runif(20, 0, 1), 2, 10)
  colnames(mT) <- nodes(g)
  rownames(mT) <- sapply(1:2, FUN = function(x) paste0("IDT", x, sep = ""))
  mT[ , 1] <- 0.9999

  mR <- t(apply(mT, MARGIN = 1, FUN = msgFGGA, matrixFGGA = mF,
                            graphOnto = g, tmax = 50, epsilon = 0.001))

  checkEquals(length(mR), 20)
  checkIdentical(rownames(mR)[1], "IDT1", "Incorrect Test")
  checkEqualsNumeric(sum(mR[,1])/2, 1, tolerance = 0.01)
}

## ----settings, include = FALSE--------------------------------------------------------------------
#library(knitr)
#opts_chunk$set(warning=TRUE, message = FALSE, cache = FALSE, tidy = FALSE, tidy.opts = list(width.cutoff = 60))
options(width = 100)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",class.source = "whiteCode")

## ----message = FALSE, eval = FALSE----------------------------------------------------------------
# ## From Bioconductor repository
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#         install.packages("BiocManager")
#     }
# BiocManager::install("fgga")
# 
# ## Or from GitHub repository using devtools
# BiocManager::install("devtools")
# devtools::install_github("fspetale/fgga")

## ----setup, eval = TRUE, message=FALSE------------------------------------------------------------
library(fgga)

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
# Loading Canis lupus familiaris dataset and example R objects
data(CfData)


## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
# To see the summarized experiment object
summary(CfData)

# To see the information of characterized data
dim(CfData$dxCf)

colnames(CfData$dxCf)[1:20]

rownames(CfData$dxCf)[1:10]

head.matrix(CfData$dxCf[, 51:61], n = 10)

# to see the information of GO data
dim(CfData$tableCfGO)

colnames(CfData$tableCfGO)[1:10]

rownames(CfData$tableCfGO)[1:10]

head(CfData$tableCfGO)[, 1:8]

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
# Checking the amount of annotations by GO-term

apply(CfData$tableCfGO, MARGIN=2, sum)

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
library(GO.db)
library(GOstats)

mygraph <- GOGraph(CfData$nodesGO, GOMFPARENTS)

# Delete root node called all
mygraph <- subGraph(CfData$nodesGO, mygraph)

# We adapt the graph to the format used by FGGA
mygraph <- t(as(mygraph, "matrix"))
mygraphGO <- as(mygraph, "graphNEL")

# We search the root GO-term
rootGO <- leaves(mygraphGO, "in")

rootGO

plot(mygraphGO)

## ----message = FALSE, eval = FALSE----------------------------------------------------------------
# # We add GO-terms corresponding to Cellular Component subdomain
# myGOs <- c(CfData[['nodesGO']], "GO:1902494", "GO:0032991", "GO:1990234",
#             "GO:0005575")
# 
# # We build a graph respecting the GO constraints of inference to MF, CC and BP subdomains
# mygraphGO <- preCoreFG(myGOs, domains="GOMF")
# 
# plot(mygraphGO)

## ----message=FALSE, include=FALSE, results='hide'-------------------------------------------------
mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

rootGO <- leaves(mygraphGO, "in")

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
modelFGGA <- fgga2bipartite(mygraphGO)

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
# We take a subset of Cf data to train our model
idsTrain <- CfData$indexGO[["indexTrain"]][1:750]

# We build our model of binary SVM classifiers
modelSVMs <- lapply(CfData[["nodesGO"]], FUN = svmTrain, 
                    tableOntoTerms = CfData[["tableCfGO"]][idsTrain, ], 
                    dxCharacterized = CfData[["dxCf"]][idsTrain, ], 
                    graphOnto = mygraphGO, kernelSVM = "radial")

## ----message = FALSE, eval = FALSE----------------------------------------------------------------
# # We calculate the reliability of each GO-term
# varianceGOs <- varianceOnto(tableOntoTerms = CfData[["tableCfGO"]][idsTrain, ],
#                         dxCharacterized = CfData[["dxCf"]][idsTrain, ],
#                         kFold = 5, graphOnto = mygraphGO, rootNode = rootGO,
#                         kernelSVM = "radial")
# 
# varianceGOs

## ----echo=FALSE, message=FALSE--------------------------------------------------------------------
CfData[["varianceGOs"]]

varianceGOs <- CfData[["varianceGOs"]]

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------

dxTestCharacterized <- CfData[["dxCf"]][CfData$indexGO[["indexTest"]][1:50], ]

matrixGOTest <- svmOnto(svmMoldel = modelSVMs, 
                    dxCharacterized = dxTestCharacterized, 
                    rootNode = rootGO, 
                    varianceSVM = varianceGOs)

head(matrixGOTest)[,1:8]

## ----message = FALSE, eval = TRUE-----------------------------------------------------------------
matrixFGGATest <- t(apply(matrixGOTest, MARGIN = 1, FUN = msgFGGA, 
                        matrixFGGA = modelFGGA, graphOnto = mygraphGO,
                        tmax = 50, epsilon = 0.001))

head(matrixFGGATest)[,1:8]

## ----message = FALSE, eval  = TRUE----------------------------------------------------------------
fHierarchicalMeasures(CfData$tableCfGO[rownames(matrixFGGATest), ], 
                        matrixFGGATest, mygraphGO)

## ----message = FALSE, eval = FALSE----------------------------------------------------------------
# # Computing F-score
# Fs <- fMeasures(CfData$tableCfGO[rownames(matrixFGGATest), ], matrixFGGATest)
# 
# # Average F-score
# Fs$perfByTerms[4]
# 
# library(pROC)
# 
# # Computing ROC curve to the first term
# rocGO <- roc(CfData$tableCfGO[rownames(matrixFGGATest), 1],  matrixFGGATest[, 1])
# 
# # Average AUC the first term
# auc(roc)
# 
# # Computing precision at different recall levels to the first term
# rocGO <- roc(CfData$tableCfGO[rownames(matrixFGGATest), 1],
#             matrixFGGATest[, 1], percent=TRUE)
# PXR <- coords(rocGO, "all", ret = c("recall", "precision"), transpose = FALSE)
# 
# # Average PxR to the first term
# apply(as.matrix(PXR$precision[!is.na(PXR$precision)]), MARGIN = 2, mean

## ----session,eval=TRUE,echo=FALSE-----------------------------------------------------------------
sessionInfo()

tableTPG <- function(att) {
    x <- as.matrix(expand.grid(rep(list(c(0, 1)), att)))
    indexTable <- seq(dim(x)[1] / 2 + 1, dim(x)[1] - 1, 1)
    x <- x[order(x[, 1]), ]
    x <- cbind(x[-indexTable, ], array(1, dim(x)[1] - length(indexTable)))
    colnames(x) <- c(
        "X_child",
        vapply(seq_len(dim(x)[2] - 2),
            FUN = function(att) {
                    (paste("X_father", att, sep = "", collapse = ""))},
            FUN.VALUE = "character"), "TPG_rule")
    x <- x + 1
    return(x)
}

library(fgga)
## create data for testing
test_check_graph <- function(){
    if(requireNamespace("graph", quietly=TRUE)){
        TRUE
    }else{
        FALSE
    }
}

test_tpg <- function(){
    x <- tableTPG(3)
    checkEquals(length(x), 20);
}

fMeasures <- function(target, predicted, cutoff = 0.5) {
    if ((nrow(target) != nrow(predicted)) || (ncol(target) != ncol(predicted)))
        stop ("Number of rows or columns do not match between target and
        predicted terms")
    predicted <- apply(predicted, c(1,2), function(x)
        {if (x >= cutoff) x <- 1 else x <- 0})
    ontoTerm <- colnames(target)
    measures <- c("Prec", "Recall", "Specif", "Fmeasure", "Acc", "nPositive")
    perfByTerms <- matrix(vapply(seq_len(ncol(target)),FUN = function(x,y,z)
        {.calculateMetrics(y[,x],z[,x])}, y=predicted, z=target,
        FUN.VALUE = numeric(6)), ncol(target), 6,
        dimnames = list(ontoTerm, measures), byrow = TRUE)

    avgTerm <- apply(perfByTerms, 2, mean)

    return(list(avgTerm=avgTerm, perfByTerms=perfByTerms))
}

fMeasuresByLevel <- function(target, predicted, graphOnto, cutoff = 0.5) {
    if (!missing(graphOnto)) {
        predicted <- apply(predicted, c(1,2), function(x)
        {if (x >= cutoff) x <- 1 else x <- 0})
        levels <- maxDistancegraphOnto(graphOnto)
        ontoTerm <- colnames(target)
        measures <- c("Prec", "Recall", "Specif", "Fmeasure", "Acc",
                    "nPositive")
        perfByTerms <- matrix(vapply(seq_len(ncol(target)),FUN = function(x,y,z)
            {.calculateMetrics(y[,x],z[,x])}, y=predicted, z=target,
            FUN.VALUE = numeric(6)), ncol(target), 6,
            dimnames = list(ontoTerm, measures), byrow = TRUE)

        gLevels <- array(sort(unique(levels)))
        perfByLevel <- matrix(0, length(gLevels), 6, dimnames=list(gLevels,
                                                                    measures))

        for (i in seq_len(length(gLevels))){
            if (gLevels[i]>0 && i==1) stop ("The graph does not have a root")
            if (gLevels[i]==0 && i==1) perfByLevel[i, ] <- apply(matrix(
                perfByTerms[which(levels == 0), ], nrow=1), 2, mean) else {
                indexLevels <- which(levels == gLevels[i])
                if (length(indexLevels) == 1) perfByLevel[i, ] <-
                    apply(t(perfByTerms[indexLevels, ]), 2, mean) else
                    perfByLevel[i, ] <- apply(
                        perfByTerms[indexLevels, ], 2, mean)
                }
            }
        } else stop ("Missing graphOnto")

    return(list(avgTerm=apply(perfByLevel, 2, mean), perfByLevel=perfByLevel))
}

.calculateMetrics <- function(termPred, termTarget) {

    if (length(termPred)!=length(termTarget))
        stop("The number of gene products per predicted term is not equal to the
            target")
    indexPosTerm <- which(termTarget > 0)
    indexNegTerm <- setdiff(seq_len(length(termTarget)), indexPosTerm)

    TP <- sum(termPred[indexPosTerm] > 0)
    FN <- sum(termPred[indexPosTerm] <= 0)
    TN <- sum(termPred[indexNegTerm] <= 0)
    FP <- sum(termPred[indexNegTerm] > 0)
    acc <- (TP+TN)/length(termTarget)

    if ((TP+FP) == 0) prec <- 0 else prec <- TP/(TP+FP)
    if ((TP+FN) == 0) recall <- 0 else recall <- TP/(TP+FN)
    if ((TN+FP) == 0)  specif <- 0 else specif <- TN/(TN+FP)
    if ((prec + recall) == 0) fMeasure <- 0 else
        fMeasure <- 2 *(prec * recall) / (prec + recall)

    measures <- c(prec,recall,specif,fMeasure,acc, length(indexPosTerm))
    names(measures) <- c("Prec", "Recall", "Specif", "Fmeasure", "Acc",
                        "nPositive")
    return (measures)
}

fHierarchicalMeasures <- function(target, predicted, graphOnto, cutoff = 0.5){
    if (dim(predicted)[2] != dim(target)[2])
        stop("The number of ontology term predicted is not equal to the target")

    commonId <- order(intersect(rownames(predicted), rownames(target)))
    target <- target[commonId, ]
    predicted <- predicted[commonId, ]
    samplesNoEval <- vector()
    gRoot <- leaves(graphOnto, "in")
    predicted <- apply(predicted, c(1,2), function(x)
        {if (x >= cutoff) x <- 1 else x <- 0})

    infoGraphBase <- list(graphOntoParents = list(),
                        graphOntoAncestors = list())
    ontoTerms <- nodes(graphOnto)
    infoGraphBase[["graphOntoParents"]] <- lapply(ontoTerms, FUN = function(x,y)
        {parents(x,y)}, y=graph_from_graphnel(graphOnto))
    names(infoGraphBase[["graphOntoParents"]]) <- ontoTerms
    infoGraphBase[["graphOntoAncestors"]] <- lapply(ontoTerms,
                    FUN = function(x,y) {ancestralSet(x,y)},
                    y=graph_from_graphnel(graphOnto))
    names(infoGraphBase[["graphOntoAncestors"]]) <- ontoTerms

    nSample <- j <- 0
    sumHR <- sumHP <- sumHF <- 0
    for (i in seq_len(dim(predicted)[1])){
        predTerms <- names(which(predicted[i,] == 1))
        targetTerms <- names(which(target[i,] == 1))
        if(length(predTerms) > 0 && length(targetTerms) > 2){
            nSample <- nSample+1
            HP <- .hPrecision(targetTerms, predTerms, graphOnto, gRoot,
                                infoGraphBase)
            HR <- .hRecall(targetTerms, predTerms, graphOnto, gRoot,
                                infoGraphBase)
            HF <- .hFmeasure(HP, HR)
            sumHP <- sumHP + HP
            sumHR <- sumHR + HR
            sumHF <- sumHF + HF
        } else {
            j<-j + 1
            samplesNoEval[j] <- rownames(predicted)[i]}
        }
    sumHP <- sumHP/nSample
    sumHR <- sumHR/nSample
    sumHF <- sumHF/nSample
    hMeasures <- list(sumHP, sumHR, sumHF, nSample, samplesNoEval)
    names(hMeasures) <- c("HP", "HR", "HF", "nSample", "noEvalSample")
    return(hMeasures)
}

.hPrecision<-function(target, predict, graphOnto, gRoot, graphOntoProperties){
    gTarget <- subGraph(target, graphOnto)
    gPred <- subGraph(predict, graphOnto)
    leafTarget <- leaves(gTarget, "out")
    leafPred <- leaves(gPred, "out")
    HP <- 0
    nLeafPred <- length(leafPred)
    nLeafTarget <- length(leafTarget)
    sumPred <- 0
    for (i in seq_len(nLeafPred)){
      parentsPred <- graphOntoProperties[["graphOntoAncestors"]][[
        leafPred[i]]]
      maxPred <- array(0,nLeafTarget)
      for (j in seq_len(nLeafTarget)){
        if(leafTarget[j] != gRoot){
          parentsTarget <- graphOntoProperties[["graphOntoAncestors"]][[
                                              leafTarget[j]]]
        }
        else parentsTarget <- gRoot
        maxPred[j] <- length(intersect(parentsPred, parentsTarget)) /
          length(parentsPred)
      }
      sumPred <- max(maxPred) + sumPred
    }
    HP <- sumPred / nLeafPred
    return(HP)
}

.hRecall<-function(target, predict, graphOnto, gRoot, graphOntoProperties){
    gTarget <- subGraph(target, graphOnto)
    gPred <- subGraph(predict, graphOnto)
    leafTarget <- leaves(gTarget, "out")
    leafPred <- leaves(gPred, "out")
    HR <- 0
    nLeafPred <- length(leafPred)
    nLeafTarget <- length(leafTarget)
    sumPred <- 0
    for (i in seq_len(nLeafTarget)){
        parentsTarget <- graphOntoProperties[["graphOntoAncestors"]][[
        leafTarget[i]]]
        maxPred <- array(0,nLeafPred)
        for (j in seq_len(nLeafPred)){
            if(leafPred[j] != gRoot){
                parentsPred <- graphOntoProperties[["graphOntoAncestors"]][[
                leafPred[j]]]
                }
        else parentsPred <- gRoot
            maxPred[j] <- length(intersect(parentsPred, parentsTarget)) /
            length(parentsTarget)
        }
        sumPred <- max(maxPred) + sumPred
    }
    HR <- sumPred / nLeafTarget
    return(HR)
}

.hFmeasure<-function(HP, HR){
    if (HP==0 && HR == 0) return (0) else return(2*HP*HR / (HP + HR))
}

library(fgga)
test_graph <- function(){
    V <- c("GO:0000166", "GO:0003674", "GO:0003676", "GO:0003677", "GO:0003824",
          "GO:0004888", "GO:0005102", "GO:1901363", "GO:1902494", "GO:1990234")
    edL <- vector("list", length=length(V))
    names(edL) <- V
    edL[[1]] <- list(edges=c(2, 3, 7, 5))
    edL[[2]] <- list(edges=c(4, 6))
    edL[[3]] <- list(edges=c(6, 8))
    edL[[4]] <- list(edges=c(6, 9))
    edL[[5]] <- list(edges=c(8))
    edL[[6]] <- list(edges=c(8, 9, 10))
    edL[[7]] <- list(edges=c(8, 5))
    edL[[8]] <- list(edges=c())
    edL[[9]] <- list(edges=c())
    g <- graph::graphNEL(nodes=V, edgeL=edL, edgemode="directed")
    return(g)
}

test_maxDistGraph <- function(){
    g <- test_graph()
    distGraph <- maxDistancegraphOnto(g)
    checkEquals(length(distGraph), length(nodes(g)))
    checkTrue(is.numeric(distGraph[1]))
}

test_measure <- function(){
    g <- test_graph()
    dxReal <- matrix(rbinom(360, 1, 0.4), 36, 10)
    colnames(dxReal) <- nodes(g)
    dxReal[, 1] <- 1
    dxTest <- matrix(runif(360, 0, 1), 36, 10)
    colnames(dxTest) <- nodes(g)
    dxTest[, 1] <- 1
    xTest <- fMeasures(dxReal, dxTest, cutoff = 0.5)
    checkIdentical(xTest[["perfByTerms"]][1, 6], 36, "Incorrect Test")
    checkEqualsNumeric(xTest[["perfByTerms"]][1, 1], 1, tolerance = 0.01)

    rownames(dxTest) <- rownames(dxReal) <- paste0("p", seq_len(36), sep="")
    hTest <- fHierarchicalMeasures(dxReal, dxTest, g, cutoff = 0.5)
    checkEquals(length(hTest), 5)
    checkTrue(is.numeric(hTest[["HP"]]))
}

fgga <- function(graphOnto, tableOntoTerms, dxCharacterized,
                dxTestCharacterized, kFold, kernelSVM = "radial", tmax = 200,
                epsilon = 0.001) {
    ontoNodes <- nodes(graphOnto)
    rootOnto <- leaves(graphOnto, "in")
    modelSVMs <- lapply(ontoNodes,
        FUN = svmTrain, tableOntoTerms = tableOntoTerms,
        dxCharacterized = dxCharacterized, graphOnto = graphOnto,
        kernelSVM = kernelSVM)

    varianceOntoTerms <- varianceOnto(tableOntoTerms, dxCharacterized,
                                kFold, graphOnto, rootOnto, kernelSVM)

    matrixOntoTest <- svmOnto(svmMoldel = modelSVMs,
                                dxCharacterized = dxTestCharacterized,
                                rootNode = rootOnto,
                                varianceSVM = varianceOntoTerms)

    modelFG <- fgga2bipartite(graphOnto)

    matrixFGGATest <- t(apply(matrixOntoTest, MARGIN = 1, FUN = msgFGGA,
        matrixFGGA = modelFG, graphOnto = graphOnto, tmax = tmax,
        epsilon = epsilon))

    return(matrixFGGATest)
}

library(fgga)
test_graph <- function(){
    V <- c("GO:0000166", "GO:0003674", "GO:0003676", "GO:0003677", "GO:0003824",
            "GO:0004888", "GO:0005102", "GO:1901363", "GO:1902494", "GO:1990234")
    edL <- vector("list", length=length(V))
    names(edL) <- V
    edL[[1]] <- list(edges=c(2, 3, 7, 5))
    edL[[2]] <- list(edges=c(4, 6))
    edL[[3]] <- list(edges=c(6, 8))
    edL[[4]] <- list(edges=c(6, 9))
    edL[[5]] <- list(edges=c(8))
    edL[[6]] <- list(edges=c(8, 9, 10))
    edL[[7]] <- list(edges=c(8, 5))
    edL[[8]] <- list(edges=c())
    edL[[9]] <- list(edges=c())
    g <- graph::graphNEL(nodes=V, edgeL=edL, edgemode="directed")
    return(g)
}

test_fgga <- function(){
    g <- test_graph()
    g <- as(g, "matrix")
    g <- rbind(g, array(0,10))
    g <- cbind(g, array(0,11))
    colnames(g)[11] <- rownames(g)[11] <- "Ontology:FES"
    g["Ontology:FES", "GO:0000166"] <- 1
    g <- as(g, "graphNEL")
    dx1 <- matrix(runif(1800, 0, 1), 360, 5)
    dx2 <- matrix(rbinom(3960, 1, 0.4), 360, 11)
    dx2[, 11] <- rep(1, 360)
    dx3 <- matrix(runif(10, 0, 1), 2, 5)
    colnames(dx1) <- colnames(dx3) <- c("AA", "BB", "CC", "DD", "EE")
    rownames(dx1) <- rownames(dx2) <- sapply(seq_len(360),
                    FUN = function(x) paste0("ID", x, sep = ""))
    rownames(dx3) <- sapply(seq_len(2),
                    FUN = function(x) paste0("IDT", x, sep = ""))
    colnames(dx2) <- nodes(g)

    mR <- fgga(graphOnto = g, tableOntoTerms = dx2, dxCharacterized = dx1,
                        dxTestCharacterized = dx3, kFold = 2,
                        tmax = 50, epsilon = 0.05)

    checkEquals(length(mR), 22)
    checkIdentical(rownames(mR)[2], "IDT2", "Incorrect Test")
    checkEqualsNumeric(sum(mR[,11])/2, 1, tolerance = 0.01)

}

varianceOnto <- function(tableOntoTerms, dxCharacterized, kFold, graphOnto,
                        rootNode, kernelSVM = "radial") {
    tableOntoTerms <- as.matrix(tableOntoTerms[
                                    order(rownames(tableOntoTerms)), ])
    dxCharacterized <- as.data.frame(dxCharacterized[order(rownames(
                                    dxCharacterized)), ])
    nodesGraph <- colnames(tableOntoTerms)
    variance <- array(0, length(nodesGraph))
    names(variance) <- nodesGraph
    for (j in seq_len(length(nodesGraph))) {
        message("Term:" ,nodesGraph[j])
        if (nodesGraph[j] != rootNode) {
            checkPosiSample <- length(which(tableOntoTerms[, j] == 1))
            if (checkPosiSample > 1) {
                if (checkPosiSample > kFold) {
                    nFold <- kFold } else { nFold <- checkPosiSample }
                dxFold <- createFolds(as.factor(tableOntoTerms[, j]), nFold)
                varianceList <- c()
                for (i in seq_len(kFold)) {
                    varianceList[i] <- .varianceSVM(indexFold = i,
                        dxFolds = dxFold, tableOntoTerms = tableOntoTerms,
                        dxCharacterized = dxCharacterized,
                        nodeOnto = nodesGraph[j],
                        graphOnto = graphOnto, kernelSVM = kernelSVM)
                }
                variance[j] <- sum(varianceList)
            } else {
                variance[j] <- 1
            }
            variance[j] <- variance[j] / (nFold)
        }
    }
    variance[which(variance < 1e-06)] <- 0.00001
    return(variance)
}

.varianceSVM <- function(indexFold, dxFolds, tableOntoTerms, dxCharacterized,
                        nodeOnto, graphOnto, kernelSVM = "radial") {
    targetOnto <- tableOntoTerms[dxFolds[[indexFold]], nodeOnto]
    dxCharacterizedValid <- dxCharacterized[dxFolds[[indexFold]], ]
    tableOntoTerms <- tableOntoTerms[-dxFolds[[indexFold]], ]
    dxCharacterized <- dxCharacterized[-dxFolds[[indexFold]], ]
    zeroValues <- array(0, 0.1 * dim(dxCharacterized)[2])
    #ancestorID <- ancestors(nodeOnto, graphOnto)
    mygraphGO <- graph_from_graphnel(graphOnto)
    ancestorID <- ancestralSet(nodeOnto, mygraphGO)
    rootNodes <- c("GO:0008150", "GO:0003674", "GO:0005575", "PO:0025131",
                    "PO:0009012", "ZFA:0100000", "HP:0000001", "Ontology:FES")
    rootNodes <- intersect(ancestorID, rootNodes)
    ancestorID <- setdiff(ancestorID, rootNodes)
    indices <- which(tableOntoTerms[, nodeOnto] == 1)
    positiveGeneNames <- rownames(tableOntoTerms)[indices]
    notNegativeGeneNames <- positiveGeneNames
    if (length(ancestorID) > 0) {
        for (i in seq_len(length(ancestorID))) {
            buffer <- rownames(tableOntoTerms)[which(tableOntoTerms[,
                                                ancestorID[i]] == 1)]
            notNegativeGeneNames <- union(notNegativeGeneNames, buffer)
        }
    }

    negativeGeneNames <- setdiff(rownames(tableOntoTerms), notNegativeGeneNames)
    np <- length(positiveGeneNames)
    nn <- length(negativeGeneNames)
    if ((np != 0) && (nn != 0)) {
        if (nn > 2 * np || np > 2 * nn) {
            if (nn > 2 * np) {
                nn <- 2 * np
                idNegative <- sample(seq_len(length(negativeGeneNames)), nn)
                exprsValues <- dxCharacterized[
                    c(positiveGeneNames, negativeGeneNames[idNegative]), ]
            } else {
                np <- 2 * nn
                idPositive <- sample(seq_len(length(positiveGeneNames)), np)
                exprsValues <- dxCharacterized[
                    c(positiveGeneNames[idPositive], negativeGeneNames), ]
            }
        } else {
            exprsValues <- dxCharacterized[c(
                positiveGeneNames, negativeGeneNames), ]
        }
        exprsValues[is.na(exprsValues)] <- 0
        gos <- as.factor(c(rep(1, np), rep(2, nn)))
        zeroValues <- apply(exprsValues, MARGIN = 2, mean)
        zeroValues <- which(zeroValues == 0)
        if (length(zeroValues) != 0) exprsValues <- exprsValues[, -zeroValues]

        model_svm <- svm(exprsValues, gos,
            method = "C-classification", cost = 500,
            kernel = kernelSVM, na.action = na.omit)

        if (length(zeroValues) != 0) {
                dxCharacterizedValid <- dxCharacterizedValid[, -zeroValues]
            }
        predOnto <- predict(model_svm, dxCharacterizedValid,
                            decision.values = TRUE)
        targetOnto[which(targetOnto == 0)] <- (-1)
        w <- which(targetOnto == 1)
        error <- sum((attr(predOnto, "decision.values")[w] -
            mean(attr(predOnto, "decision.values")[w]))^2)
        variancePos <- error / length(w)
        error <- sum((attr(predOnto, "decision.values")[-w] -
            mean(attr(predOnto, "decision.values")[-w]))^2)
        varianceNeg <- error / (length(predOnto) - length(w))
    } else { variancePos <- 0 }
    return(variancePos)
}

---
title: "FGGA: Factor Graph GO Annotation"
author:
- name: F.E. Spetale
  affiliation: 
  - &id Bioinformatics Group. CIFASIS-UNR-CONICET. Rosario (Argentina)
  email: spetale@cifasis-conicet.gov.ar
- name: E. Tapia
  affiliation: *id
package: fgga
date: "`r Sys.Date()`"
output:
    BiocStyle::html_document:
    toc_float: true
vignette: >
    %\VignetteIndexEntry{FGGA: Factor Graph GO Annotation}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}

abstract: We present a tutorial to use **FGGA**, a hierarchical ensemble method that exploits the ability of factor graphs for modeling logical and statistical constraints over system variables. The **FGGA model** is built in two main steps. In the first step, a core Factor Graph (FG) modeling hidden GO-term predictions and relationships is created. In the second step, the FG is enriched with nodes modeling observable GO-term predictions issued by **binary SVM classifiers**. In addition, probabilistic constraints modeling learning gaps between hidden and observable GO-term predictions are introduced. These gaps are assumed to be independent among GO-terms, locally additive with respect to observed predictions, and zero-mean Gaussian. **FGGA predictions** are issued by the native iterative **message passing algorithm** of factor graphs.
---

```{css, echo = FALSE, eval = TRUE}
.whiteCode {
    background-color: white;
    border-color: #337ab7 !important;
    border: 1px solid;
}
```

```{r settings, include = FALSE}
#library(knitr)
#opts_chunk$set(warning=TRUE, message = FALSE, cache = FALSE, tidy = FALSE, tidy.opts = list(width.cutoff = 60))
options(width = 100)
knitr::opts_chunk$set(collapse = TRUE, comment = "#>",class.source = "whiteCode")
```

# Automated Gene Ontology (GO) annotation methods

As volume of genomic data grows, computational methods become essential for providing a first glimpse onto gene annotations. Automated Gene Ontology (GO) annotation methods based on hierarchical ensemble classification techniques are particularly interesting when interpretability of annotation results is a main concern. In these methods, raw GO-term predictions computed by base binary classifiers are leveraged by checking the consistency of predefined GO relationships. FGGA is a factor graph approach to the hierarchical ensemble formulation of the automated GO annotation problem. In this formal method, a core factor graph is first built based on the GO structure and then enriched to take into account the noisy nature of GO-term predictions. Hence, starting from raw GO-term predictions, an iterative message passing algorithm between nodes of the factor graph is used to compute marginal probabilities of target GO-terms.

The method is detailed in the original [publications](#references) [1, 2], but this brief **vignette** explains how to use **FGGA** to predict a set of GO-terms (GO node labels) from gene-product sequences of a given organism. The aim is to improve the quality of existing electronic annotations and provide a new annotation for those unknown sequences that can not be classified by traditional methods such as Blast.

Thus, **FGGA** is a tool to the automated annotation of protein sequences, however, the annotation of other types of striking functional gene products is also possible, e.g., long non-coding RNAs. FGGA may bring an opportunity for improving the annotation of long non-coding RNA sequences through boosting the confidence of base binary classifiers by the characterization of their structures primary, secondary and tertiary. Along the vignette, we used protein coding genes data from Cannis familiaris organism obtained with [UniProt](https://www.uniprot.org/uniprot/?query=taxonomy:9615).

![Fig. 1 - Workflow of FGGA algorithm.](framework.png)

# Installation

The **fgga** R source package can be downloaded from [**Bioconductor repository**](https://bioconductor.org/) or [**GitHub repository**](https://github.com/fspetale/fgga). This R package contains a experimental dataset as example, one pre-run R object and all functions needed to run FGGA.

```{r, message = FALSE, eval = FALSE}
## From Bioconductor repository
if (!requireNamespace("BiocManager", quietly = TRUE)) {
        install.packages("BiocManager")
    }
BiocManager::install("fgga")

## Or from GitHub repository using devtools
BiocManager::install("devtools")
devtools::install_github("fspetale/fgga")
```

After the package is installed, it can be loaded into R workspace by

```{r setup, eval = TRUE, message=FALSE}
library(fgga)
```

# Input data

At present, the method directly supports data characterized from gene product sequences. This characterization can be generated according to the expert's criteria. For example, possible characterizations can be: [PFAM motifs](https://pfam.xfam.org/), [physico-chemical properties](https://web.expasy.org/protparam/), [signal peptides](http://www.cbs.dtu.dk/services/), among others. The admitted values can be of the numeric, boolean or character type. However, for greater efficiency of the binary classification algorithm it is recommended that the use of normalized numerical values. These datasets must have at least 50 annotations per GO-term to train the FGGA model.

# An example of the usage of FGGA for the automated GO annotation

In this section, we apply **FGGA** to predict the biological functions, GO-terms, of *Canis lupus familiaris* proteins. We download 6962 *Canis lupus familiaris*, Cf, protein sequences with their GO annotations from [UniProt](https://www.uniprot.org/uniprot/?query=taxonomy:9615) and then were characterized through physico-chemical properties from R package [Peptides](https://cran.r-project.org/web/packages/Peptides).

## Data Loading

Let us import the toy dataset in the workspace:

```{r, message = FALSE, eval = TRUE}
# Loading Canis lupus familiaris dataset and example R objects
data(CfData)

```

```{r, message = FALSE, eval = TRUE}
# To see the summarized experiment object
summary(CfData)

# To see the information of characterized data
dim(CfData$dxCf)

colnames(CfData$dxCf)[1:20]

rownames(CfData$dxCf)[1:10]

head.matrix(CfData$dxCf[, 51:61], n = 10)

# to see the information of GO data
dim(CfData$tableCfGO)

colnames(CfData$tableCfGO)[1:10]

rownames(CfData$tableCfGO)[1:10]

head(CfData$tableCfGO)[, 1:8]
```

*tableCfGO* is a binary matrix whose $(i, j)_{th}$ component is 1 if protein $i$ is annotated with $GO-term_j$, 0 otherwise. Note that the row names of both dxCf and tableCfGO are identical. Our binary classifiers require at least 50 annotations per GO-terms. Therefore, this condition is checked and those GO-terms that do not comply are eliminated.

```{r, message = FALSE, eval = TRUE}
# Checking the amount of annotations by GO-term

apply(CfData$tableCfGO, MARGIN=2, sum)
```

## GO subgraph building

If we want to predict GO-terms in a single subdomain, *BP, MF or CC* , we must build the GO-DAG associated with these GO-terms.

```{r, message = FALSE, eval = TRUE}
library(GO.db)
library(GOstats)

mygraph <- GOGraph(CfData$nodesGO, GOMFPARENTS)

# Delete root node called all
mygraph <- subGraph(CfData$nodesGO, mygraph)

# We adapt the graph to the format used by FGGA
mygraph <- t(as(mygraph, "matrix"))
mygraphGO <- as(mygraph, "graphNEL")

# We search the root GO-term
rootGO <- leaves(mygraphGO, "in")

rootGO

plot(mygraphGO)
```

On the other hand, if you want to predict in two or three subdomains you should use the preCoreFG function. This function builds a graph respecting the GO constraints of inference and also links the GO-terms of the selected subdomains.

```{r, message = FALSE, eval = FALSE}
# We add GO-terms corresponding to Cellular Component subdomain
myGOs <- c(CfData[['nodesGO']], "GO:1902494", "GO:0032991", "GO:1990234", 
            "GO:0005575")

# We build a graph respecting the GO constraints of inference to MF, CC and BP subdomains
mygraphGO <- preCoreFG(myGOs, domains="GOMF")

plot(mygraphGO)
```

![Fig. 3 - GO-Plus subgraph.](goPlus.png)

## Matching a GO-DAG to a Factor Graph

Let's a GO subgraph, GO-terms [*GO:i*](GO:i){.uri} are mapped to binary-valued variable nodes $x_i$ of a factor graph while relationships between GO-terms are mapped to logical factor nodes $f_k$ describing valid [*GO:i*](GO:i){.uri} configurations under the True Path Graph constraint.

```{r message=FALSE, include=FALSE, results='hide'}
mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

rootGO <- leaves(mygraphGO, "in")
```

```{r, message = FALSE, eval = TRUE}
modelFGGA <- fgga2bipartite(mygraphGO)
```

## Flat prediction with FGGA clasiffier

Now, let's use the MF subgraph to build our model of binary SVM classifiers with a training set of the Cf data.

```{r, message = FALSE, eval = TRUE}
# We take a subset of Cf data to train our model
idsTrain <- CfData$indexGO[["indexTrain"]][1:750]

# We build our model of binary SVM classifiers
modelSVMs <- lapply(CfData[["nodesGO"]], FUN = svmTrain, 
                    tableOntoTerms = CfData[["tableCfGO"]][idsTrain, ], 
                    dxCharacterized = CfData[["dxCf"]][idsTrain, ], 
                    graphOnto = mygraphGO, kernelSVM = "radial")
```

```{r, message = FALSE, eval = FALSE}
# We calculate the reliability of each GO-term
varianceGOs <- varianceOnto(tableOntoTerms = CfData[["tableCfGO"]][idsTrain, ], 
                        dxCharacterized = CfData[["dxCf"]][idsTrain, ], 
                        kFold = 5, graphOnto = mygraphGO, rootNode = rootGO, 
                        kernelSVM = "radial")

varianceGOs
```

```{r echo=FALSE, message=FALSE}
CfData[["varianceGOs"]]

varianceGOs <- CfData[["varianceGOs"]]
```

Next, we predict each GO-terms from a test set using our model of binary SVM classifiers.

```{r, message = FALSE, eval = TRUE}

dxTestCharacterized <- CfData[["dxCf"]][CfData$indexGO[["indexTest"]][1:50], ]

matrixGOTest <- svmOnto(svmMoldel = modelSVMs, 
                    dxCharacterized = dxTestCharacterized, 
                    rootNode = rootGO, 
                    varianceSVM = varianceGOs)

head(matrixGOTest)[,1:8]
```

## Compute marginal probabilities of GO-terms by each protein

Once a factor graph model *FG* for the automated GO annotation problem has been defined, an iterative message passing algorithm between nodes of *FG* can be used to compute maximum a posteriori (MAP) estimates of variable nodes $x_i$ modeling actual [*GO:i*](GO:i){.uri} annotations.

The function *msgFGGA* returns a matrix with *k* rows and *m* columns corresponding to the Cf proteins and MF GO-terms respectively.

```{r message = FALSE, eval = TRUE}
matrixFGGATest <- t(apply(matrixGOTest, MARGIN = 1, FUN = msgFGGA, 
                        matrixFGGA = modelFGGA, graphOnto = mygraphGO,
                        tmax = 50, epsilon = 0.001))

head(matrixFGGATest)[,1:8]
```

# FGGA Performance

We now evaluate the performance of **FGGA** in terms of hierarchical F-score. The hierarchical classification performance metrics like the hierarchical precision (HP), the hierarchical recall (HR), and the hierarchical F-score (HF) measures [publications](#references) [3] properly recognize partially correct classifications and correspondingly penalize more distant or more superficial errors. The formulas of the hierarchical metrics are shown below:

$$ 
HP(s) = \frac{1}{\mid l(P_{G}(s)) \mid} \hspace{1.25mm} \sum_{q \hspace{0.5mm} \in \hspace{0.5mm} l(P_{G}(s))} \hspace{1.5mm} \max_{c \hspace{0.5mm} \in \hspace{0.5mm} l(C_{G}(s))}  \frac{\mid \uparrow c \hspace{1mm} \cap \uparrow q \mid}{\uparrow q} 
$$

$$ HR(s) = \frac{1}{\mid l(C_{G}(s)) \mid} \hspace{1.25mm} \sum_{c \hspace{0.5mm} \in \hspace{0.5mm} (C_{G}(s))} \hspace{1.5mm} \max_{q \hspace{0.5mm} \in \hspace{0.5mm} l(P_{G}(s))}  \frac{\mid \uparrow c \hspace{1mm} \cap \uparrow q \mid}{\uparrow c}
$$

$$ HF(s) = \frac{2 \cdot HP \cdot HR}{HP + HR}
$$

```{r, message = FALSE, eval  = TRUE}
fHierarchicalMeasures(CfData$tableCfGO[rownames(matrixFGGATest), ], 
                        matrixFGGATest, mygraphGO)
```

Finally, we evaluate the performance of *FGGA* in terms of Area under the ROC Curve (AUC) and in terms of Precision x Recall (PxR). We use the R package [pROC](https://cran.r-project.org/web/packages/pROC), which provides functions to compute the performance measures we need.

```{r, message = FALSE, eval = FALSE}
# Computing F-score
Fs <- fMeasures(CfData$tableCfGO[rownames(matrixFGGATest), ], matrixFGGATest)

# Average F-score
Fs$perfByTerms[4]

library(pROC)

# Computing ROC curve to the first term
rocGO <- roc(CfData$tableCfGO[rownames(matrixFGGATest), 1],  matrixFGGATest[, 1])

# Average AUC the first term
auc(roc)

# Computing precision at different recall levels to the first term
rocGO <- roc(CfData$tableCfGO[rownames(matrixFGGATest), 1],  
            matrixFGGATest[, 1], percent=TRUE)
PXR <- coords(rocGO, "all", ret = c("recall", "precision"), transpose = FALSE)

# Average PxR to the first term
apply(as.matrix(PXR$precision[!is.na(PXR$precision)]), MARGIN = 2, mean
```

```{r session,eval=TRUE,echo=FALSE} 
sessionInfo()
```

# References {#references}

1: Spetale F.E., Tapia E., Krsticevic F., Roda F. and Bulacio P. "A Factor Graph Approach to Automated GO Annotation". PLoS ONE 11(1): e0146986, 2016.

2: Spetale Flavio E., Arce D., Krsticevic F., Bulacio P. and Tapia E. "Consistent prediction of GO protein localization". Scientific Report 7787(8), 2018

3: Verspoor K., Cohn J., Mnizewski S., Joslyn C. "A categorization approach to automated ontological function annotation". Protein Science. 2006;15:1544--1549

\name{fgga}
\alias{fgga}

\title{
Factor Graph Cross-Ontology Annotation model
}
\description{
A hierarchical graph-based machine learning model for the consistent GO, PO, ZFA, HPO
annotation of protein coding genes.
}
\usage{
fgga(graphOnto, tableOntoTerms, dxCharacterized, dxTestCharacterized,
    kFold, kernelSVM, tmax, epsilon)
}

\arguments{
    \item{graphOnto}{A graphNEL graph with `m' Ontology node labels.}

    \item{tableOntoTerms}{A binary matrix with `n' proteins (rows) by `m' Ontology node labels (columns).}

    \item{dxCharacterized}{A data frame with `n' proteins (rows) by `f' features (columns).}

    \item{dxTestCharacterized}{A data frame with `k' proteins (rows) by `f' features (columns).}

    \item{kFold}{An integer for the number of folds.}

    \item{kernelSVM}{The kernel used to calculate the variance (default: radial).}

    \item{tmax}{An integer indicating the maximum number of iterations (default: 200).}

    \item{epsilon}{A real value less than 1 that represents the convergence criteria (default: 0.001).}
}

\details{
The \bold{FGGA model} is built in two main steps. In the first step, a core Factor Graph (FG) modeling hidden Ontology-term predictions and relationships is created. In the second step, the FG is enriched with nodes modeling observable Ontology-term predictions issued by \bold{binary SVM classifiers}. In addition, probabilistic constraints modeling learning gaps between hidden and observable Ontology-term predictions are introduced. These gaps are assumed to be independent among Ontology-terms, locally additive with respect to observed predictions, and zero-mean Gaussian. \bold{FGGA predictions} are issued by the native iterative \bold{message passing algorithm} of factor graphs.
}

\value{
A named matrix with `k' protein coding genes (rows) by `m' cross-Ontology node labels (columns) where each element indicates a probabilistic prediction value.
}

\references{
Spetale F.E., Tapia E., Krsticevic F., Roda F. and Bulacio P. “A Factor Graph Approach to Automated GO Annotation”. PLoS ONE 11(1): e0146986, 2016.

Spetale Flavio E., Arce D., Krsticevic F., Bulacio P. and Tapia E. “Consistent prediction of GO protein localization”. Scientific Report 7787(8), 2018
}

\author{
Flavio E. Spetale and Elizabeth Tapia <spetale@cifasis-conicet.gov.ar>
}

\seealso{
\code{\link[fgga]{fgga2bipartite}}, \code{\link[fgga]{sumProduct}}, \code{\link[fgga]{svmOnto}}
}
\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

dxCfTestCharacterized <- CfData[["dxCf"]][CfData[["indexGO"]]$indexTest[1:2], ]

myTableGO <- CfData[["tableCfGO"]][
                    CfData[["indexGO"]]$indexTrain[1:300], ]

dataTrain <- CfData[["dxCf"]][
                    CfData[["indexGO"]]$indexTrain[1:300], ]

fggaResults <- fgga(graphOnto = mygraphGO,
                tableOntoTerms = myTableGO, dxCharacterized = dataTrain,
                dxTestCharacterized = dxCfTestCharacterized, kFold = 2,
                tmax = 50, epsilon = 0.05)

}

\keyword{fgga2bipartite}
\keyword{msgFGGA}

\name{preCoreFG}
\alias{preCoreFG}
\title{Transitive closure processing of a cross-ontology DAG}
\description{
preCoreFG ensures the transitive closure of inference paths -serial concatenation of relationships- in a cross-ontology DAG.}

\usage{
preCoreFG(ontoTerms, domains = "GO")
}

\arguments{
    \item{ontoTerms}{A vector with `m' cross-ontology node labels}

    \item{domains}{A string that indicates which subdomains or ontologies will be used. Values: ``GOBP", ``GOMF", ``GOCC", ``GOCC-PO", ``GOCC-ZFA", ``GOBP-HPO", ``GOMF-HPO", ``GOCC-HPO", ``GO-PO", ``GO-ZFA", ``GO-HPO", ``GO" (default, ``BP-MF-CC")}

}

\details{
Non-transitive relationships in cross-ontology DAG's may lead to non-transitive inference paths precluding the free propagation and consistency checking of ontology annotations. A transitive closure screening process over cross-ontology DAG's relationships is required before the construction of Forney Factor Graphs. Serial concatenation of relationships leading to non-transitive inference paths in a cross-ontology DAG are conformed by removing the most specific relationship.
}

\value{
A graphNEL graph with `m' node labels belong to ontologies used.
}

\references{
Spetale Flavio E., Arce D., Krsticevic F., Bulacio P. and Tapia E. “Consistent prediction of GO protein localization”. Scientific Report 7787(8), 2018
}
\author{
Flavio E. Spetale <spetale@cifasis-conicet.gov.ar>
}

\seealso{
\code{\link[fgga]{fgga2bipartite}}
}
\examples{

data(CfData)

myGOs <- c(CfData[["nodesGO"]], "GO:1902494", "GO:0032991", "GO:1990234",
            "GO:0005575")

\donttest{
# mygraphGO <- preCoreFG(myGOs, domains = "GOMF")
}}

\keyword{msgFFGA}

\name{svmOnto}
\alias{svmOnto}
\title{
Ontology-term predictions by binary SVM classifiers
}
\description{
svmOnto delivers soft Ontology-term predictions based on binary SVM classification models.
}
\usage{
svmOnto(svmMoldel, dxCharacterized, rootNode, varianceSVM)
}
\arguments{

    \item{svmMoldel}{A list of object of class ``svm" created by svm.}

    \item{dxCharacterized}{A data frame with `n' protein coding genes (rows) by `f' features (columns).}

    \item{rootNode}{A character indicating the root of the graph.}

    \item{varianceSVM}{A vector named with the variance of cross-Ontology node labels.}
}
\details{
Binary SVM predictions are supplemented with their corresponding margins. These margins are used to model the additive zero-mean Gaussian learning noise that corrupts ideal but hidden Ontology-term predictions. These ideal predictions are embedded in hidden variable nodes of the Forney Factor Graph.
}
\value{
    \item{svmOnto}{A named vector of predicted values for a protein sequence.}
}
\references{
Chang, Chih-Chung and Lin, Chih-Jen: LIBSVM: a library for Support Vector Machines
\url{http://www.csie.ntu.edu.tw/~cjlin/libsvm}

Eisner R, Poulin B, Szafron D, Lu P, Greiner R. Improving protein function prediction using the hierarchical structure of the Gene Ontology. In: Proc. IEEE CIBCB; 2005. p. 1–1

Spetale FE, Tapia E, Krsticevic F, Roda F, Bulacio P (2016). A Factor Graph Approach to Automated GO Annotation. PLOS ONE 11(1): e0146986
}
\author{
Flavio E. Spetale, Pilar Bulacio and Javier Murillo <spetale@cifasis-conicet.gov.ar>
}

\seealso{
\code{\link[fgga]{svmTrain}}
}
\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

modelSVMs <- lapply(CfData[["nodesGO"]][1:4], FUN = svmTrain,
                    tableOntoTerms = CfData[["tableCfGO"]],
                    dxCharacterized = CfData[["dxCf"]],
                    graphOnto = mygraphGO, kernelSVM = "radial")

rootGO <- leaves(mygraphGO, "in")

varianceGOs <- CfData[["varianceGOs"]]

# SVM testing in four GO-terms
dxTestCharacterized <- CfData[["dxCf"]][
                                sample(1:dim(CfData[["dxCf"]])[1], 20), ]

matrixGOTest <- svmOnto(svmMoldel = modelSVMs,
                        dxCharacterized = dxTestCharacterized,
                        rootNode = rootGO, varianceSVM = varianceGOs)
}

\keyword{svm}

\name{svmTrain}
\alias{svmTrain}
\title{
Binary SVM classification models for individual Ontology-term predictions
}
\description{
svmTrain delivers a set of binary SVM classifiers for different Ontology-terms.
}
\usage{
svmTrain(nodeGraph, tableOntoTerms, dxCharacterized, graphOnto,
        kernelSVM = "radial")
}

\arguments{
    \item{nodeGraph}{A character indicating a GO node label}

    \item{tableOntoTerms}{A binary matrix with `n' proteins (rows) by `m' Ontology node labels (columns).}

    \item{dxCharacterized}{A data frame with `n' protein coding genes (rows) by `f' features (columns).}

    \item{graphOnto}{A graphNEL graph with `m' Ontology node labels.}

    \item{kernelSVM}{The kernel used to calculate the variance (default: radial).}
}

\details{
Starting from sets of positively annotated protein sequences to different GO-terms in a GO subgraph, corresponding sets of negatively annotated protein sequences are computed using the inclusive separation policy proposed by Eisner et al. Training datasets for each GO-term are used to train binary Support Vector Machine (SVM) classifiers with a variety of kernel options.
}
\value{
    \item{svmTrain}{A list of objects of ``svm" class containing the fitted model.}
}

\references{
Chang, Chih-Chung and Lin, Chih-Jen: LIBSVM: a library for Support Vector Machines
\url{http://www.csie.ntu.edu.tw/~cjlin/libsvm}

Eisner R, Poulin B, Szafron D, Lu P, Greiner R. Improving protein function prediction using the hierarchical structure of the Gene Ontology. In: Proc. IEEE CIBCB; 2005. p. 1–1

Spetale FE, Tapia E, Krsticevic F, Roda F, Bulacio P (2016). A Factor Graph Approach to Automated GO Annotation. PLOS ONE 11(1): e0146986
}
\author{
Flavio E. Spetale, Pilar Bulacio and Javier Murillo <spetale@cifasis-conicet.gov.ar>
}

\seealso{
\code{\link[fgga]{svmOnto}}
}
\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

# SVM training in four GO-terms
modelSVMs <- lapply(CfData[["nodesGO"]][1:4], FUN = svmTrain,
                    tableOntoTerms = CfData[["tableCfGO"]],
                    dxCharacterized = CfData[["dxCf"]],
                    graphOnto = mygraphGO, kernelSVM = "radial")

}

\keyword{svm}

\name{varianceOnto}
\alias{varianceOnto}
\title{
The variance of the gaussian learning noise at individual Ontology-terms
}
\description{
varianceOnto estimates the variance of gaussian distributions modeling the additive learning noise that corrupts ideal Ontology-term predictions.
}
\usage{
varianceOnto(tableOntoTerms, dxCharacterized, kFold, graphOnto, rootNode,
            kernelSVM = "radial")
}
\arguments{
    \item{tableOntoTerms}{A binary matrix with `n' protein coding genes (rows) by `m' cross-Ontology node labels (columns).}

    \item{dxCharacterized}{A data frame with `n' protein coding genes (rows) by `f' features (columns).}

    \item{kFold}{An integer for the number of folds.}

    \item{graphOnto}{A graphNEL graph with `m' cross-Ontology node labels.}

    \item{rootNode}{A character indicating the root of the graph.}

    \item{kernelSVM}{The kernel used to calculate the variance (default: radial).}
}
\details{
Under the assumption of symmetrical (Gaussian) conditional probability distributions for observable variable node predictions \eqn{y_i} over a hidden variable node annotations \eqn{x_i}, variances \eqn{\eta_i} can be estimated using a validation dataset of positively annotated samples. \if{latex}{\out{Let \emph{D} be a validation dataset with $L^{+}$ positively annotated samples
\deqn{\hat{\eta_i} = 1/(L^{+} - 1) * \sum_{i=1}^L (x_i - y_i)}

where $x_i = 1$ is the positive annotation of the l-th data sample to the ith Ontology-term and $y_i$  is the corresponding real-valued classifier (SVM) prediction.}}
}

\value{
A vector named with the variance of each cross-Ontology node.}

\references{
Spetale FE, Tapia E, Krsticevic F, Roda F, Bulacio P (2016). A Factor Graph Approach to Automated GO Annotation. PLOS ONE 11(1): e0146986
}
\author{
Flavio E. Spetale <spetale@cifasis-conicet.gov.ar>
}

\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

rootGO <- leaves(mygraphGO, "in")

mygraphGO <- subGraph(c("GO:0140110", "GO:0098772", "GO:0003674"), mygraphGO)

myTableGO <- CfData[["tableCfGO"]][
                    CfData[["indexGO"]]$indexTrain,
                    c("GO:0140110", "GO:0098772", "GO:0003674")]

varianceGOs <- varianceOnto(tableOntoTerms = myTableGO,
                            dxCharacterized = CfData[["dxCf"]],
                            kFold = 2, graphOnto = mygraphGO,
                            rootNode = rootGO, kernelSVM = "radial")
}

\keyword{variance}

\name{fgga2bipartite}
\alias{fgga2bipartite}
\title{
Forney Factor Graph model
}
\description{
fgga2bipartite builds a Forney Factor Graph from a FGGA model.
}
\usage{
fgga2bipartite(graphOnto)
}

\arguments{
    \item{graphOnto}{A graphNEL graph with `m' cross-Ontology node labels.}
}

\details{
The \bold{Gene Ontology} (GO) is structured as a directed acyclic graph (DAG) with nodes (GO-terms) representing gene functions and edges characterizing relationships between nodes. A variety of relationships are possible (currently 8). To compute GO-term predictions perfectly aware of GO-term relationships, a  Forney Factor Graph is required. Hence, GO-terms are mapped to binary variable nodes, and relationships to logical factor nodes.
}

\value{
A binary matrix with \emph{2*m} rows by \emph{2*m-1} columns where \emph{m} is the quantity of cross-Ontology node labels.
}

\references{
F. Spetale, J. Murillo, E. Tapia, D. Arce, S. Ponce, and P. Bulacio, “Formal modeling of gene ontology annotation predictions based on factor graphs,” Journal of Physics: Conference Series, vol. 705, no. 1, p. 012001, 2016.

Spetale F.E., Tapia E., Krsticevic F., Roda F. and Bulacio P. “A Factor Graph Approach to Automated GO Annotation”. PLoS ONE 11(1): e0146986, 2016.

Spetale Flavio E., Arce D., Krsticevic F., Bulacio P. and Tapia E. “Consistent prediction of GO protein localization”. Scientific Report 7787(8), 2018
}

\author{
Flavio E. Spetale <spetale@cifasis-conicet.gov.ar>
}

\examples{
data(CfData)

graphGO <- as(CfData$graphCfGO, "graphNEL")
fgga2bipartite(graphGO)
}

\name{sumProduct}
\alias{msgFGGA}
\title{
Message passing algorithm between nodes of the Forney Factor Graph
}
\description{
msgFGGA operates in Forney Factor Graphs and computes approximate maximum a posteriori (MAP) estimates of hidden Ontology variable nodes (Ontology-terms).
}

\usage{
msgFGGA(matrixFGGA, obsValueOntoTerms, graphOnto, tmax = 200,
        epsilon = 0.001)
}

\arguments{
    \item{matrixFGGA}{A binary matrix with FGGA model of the class `fgga.'}

    \item{obsValueOntoTerms}{A named vector with `m' probabilistic prediction values for a protein coding gene.}

    \item{graphOnto}{A graphNEL graph with `m' Ontology node labels.}

    \item{tmax}{An integer indicating the maximum number of iterations (defualt: 200).}

    \item{epsilon}{An integer that represents the convergence criteria (default: 0.001)}
}

\details{
Starting from Ontology-term predictions at observable variable nodes, probability distribution functions modelling the learning noise of individual Ontology-terms, a user-defined number of iterations (maximum 200), a user-defined threshold for the convergence of predictions (maximum 0.001), and the structure of the Forney Factor Graph, the \bold{msgFGGA} delivers approximate maximum a posteriori (MAP) estimates of hidden GO variable nodes (GO-terms).
}

\value{
A named vector with `m' consistent probabilistic predictions for a protein coding genes.
}
\references{
Kschischang FR, Frey BJ, Loeliger H.-A. Factor graphs and the sum-product algorithm. IEEE Trans. Inf. Theor. 47, 498–519 (2001).

Yedidia JS. Message-passing algorithms for inference and optimization. Journal of Statistical Physics 145, 860–890 (2011).

Spetale FE, Tapia E, Krsticevic F, Roda F, Bulacio P (2016). A Factor Graph Approach to Automated GO Annotation. PLOS ONE 11(1): e0146986
}
\author{
Flavio E. Spetale and Elizabeth Tapia <spetale@cifasis-conicet.gov.ar>
}

\seealso{
\code{\link[fgga]{tableTPG}}
}
\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

myTableGO <- CfData[["tableCfGO"]][
                    CfData[["indexGO"]]$indexTrain[1:500], ]

modelSVMs <- lapply(CfData[["nodesGO"]], FUN = svmTrain,
                    tableOntoTerms = myTableGO,
                    dxCharacterized = CfData[["dxCf"]],
                    graphOnto = mygraphGO, kernelSVM = "radial")

rootGO <- leaves(mygraphGO, "in")

varianceGOs <- CfData[["varianceGOs"]]

dxTestCharacterized <- CfData[["dxCf"]][
                                sample(1:dim(CfData[["dxCf"]])[2], 2), ]

matrixGOTest <- svmOnto(svmMoldel = modelSVMs,
                        dxCharacterized = dxTestCharacterized,
                        rootNode = rootGO, varianceSVM = varianceGOs)

modelFGGA <- fgga2bipartite(mygraphGO)

matrixFGGATest <- t(apply(matrixGOTest, MARGIN = 1, FUN = msgFGGA,
                    matrixFGGA = modelFGGA, graphOnto= mygraphGO,
                    tmax = 50, epsilon = 0.1))

}

\keyword{msgFGGA}

\name{fgga-package}
\alias{fgga-package}
\docType{package}
\title{FGGA: Factor Graph Gene ontology Annotation.}
\description{
FGGA is a graph-based machine learning approach for the automated and consistent GO, PO, HPO and ZFA annotation of protein coding genes. The input is a set of ontoligical-terms annotated protein coding genes previously characterized in terms of a fixed number of user-defined features, including the presence/absence of PFAM domains, physical-chemical properties, presence of signal peptides, among others. The set of ontoligical terms defines the output cross-ontology subgraph. A hierarchical ensemble (SVMs) machine learning model is generated. This model can be used to predict the cross-ontology subgraph annotations of uncharacterized protein coding genes. Individual ontoligical-term annotations are accompanied by maximum a posteriori probability estimates issued by the native message passing algorithm of factor graphs.
}

\author{
\emph{Flavio E. Spetale}, \emph{Paolo Cacchiarrelli} and \emph{Elizabeth Tapia}

BioInformatics

Cifasis-Conicet

\email{spetale@cifasis-conicet.gov.ar}

Maintainer:
\emph{Flavio E. Spetale}
}

\references{
Spetale F.E., et al. \bold{A Factor Graph Approach to Automated GO Annotation}. \emph{PLoS ONE} (2016). \url{ https://doi.org/10.1371/journal.pone.0146986}.

Spetale Flavio E., et al. \bold{Consistent prediction of GO protein localization}. \emph{Scientific Report} (2018). \url{https://doi.org/10.1038/s41598-018-26041-z}.

}
\keyword{ package }
\seealso{
\code{\link[fgga]{fgga}}, \code{\link[fgga]{fgga2bipartite}}, \code{\link[fgga]{sumProduct}}, \code{\link[e1071]{svm}}
}

\name{createFolds}
\alias{createFolds}
\title{
Data splitting function useful for binary classification tasks
}
\description{
createFolds splits binary classification data into k-folds.
}

\usage{
createFolds(target, k_fold = 10)
}

\arguments{
    \item{target}{A binary vector of a Ontology class}

    \item{k_fold}{An integer for the number of folds}
}
\details{
A random sampling is performed on binary classification data. A set of k data folds reflecting the original class balance is obtained.
}
\value{
list of row position integers corresponding to the training data
}
\references{
Hyndman and Athanasopoulos (2013), Forecasting: principles and practice. \url{https://www.otexts.org/fpp}
}
\author{
Flavio E. Spetale and Pilar Bulacio <spetale@cifasis-conicet.gov.ar>
}

\examples{
data(CfData)

createFolds(CfData[["tableCfGO"]][ ,"GO:0005515"], k_fold = 2)}

\name{fMeasure}
\alias{fMeasures}
\alias{fHierarchicalMeasures}
\alias{fMeasuresByLevel}

\title{
Individual and hierarchical F-measures
}
\description{
Set of functions to compute the individual and hierarchical F-score, precision, recall.
}
\usage{
    fMeasures(target, predicted, cutoff = 0.5)
    fMeasuresByLevel(target, predicted, graphOnto, cutoff = 0.5)
    fHierarchicalMeasures(target, predicted, graphOnto, cutoff = 0.5)
}

\arguments{
    \item{target}{
    A binary matrix with `n' proteins (rows) by `m' Ontology node labels (columns) corresponding to the target of ontology terms where 0 stands for negative and 1 for positive.
    }
    \item{predicted}{
    A real matrix with `n' proteins (rows) by `m' Ontology node labels (columns) corresponding to the predicted terms.
    }
    \item{graphOnto}{
    A graphNEL graph with `m' Ontology node labels.
    }
    \item{cutoff}{
    A real value to divide the predicted terms into positive and negative. The predicted values higher than the cutoff will be taken as positive.
    }
}

\details{
    \code{fMeasures} computes the F-score, precision, recall, specificity and accuracy for each ontological term.

    \code{fMeasuresByLevel} computes F-score, precision, recall, specificity and accuracy for all ontological terms belongs to graph. The levels are calculated as the maximum distance between two terms of the graph.

    \code{fHierarchicalMeasures} computes the hierarchical F-score, precision, recall for the predicted terms of a set of proteins.
}

\value{
    \code{fMeasures} and \code{fMeasuresByLevel} returns a list of two elements where the first element is a named vector with six attributes while the second element is an array of 'm' ontological terms by six attributes. The 6 attributes are:
    \item{Prec:}{Precision}
    \item{Recall:}{Recall}
    \item{Specif:}{Specificity}
    \item{Fmeasure:}{F-score}
    \item{Acc:}{Accuracy}
    \item{nPositive:}{Number of positive samples}

    \code{fHierarchicalMeasures} returns a list of five elements:
    \item{HP:}{Hierarchical Precision}
    \item{HR:}{Hierarchical Recall}
    \item{HF:}{Hierarchical F-score}
    \item{nSample: }{Number of proteins evaluated}
    \item{noEvalSample: }{Named vector of proteins not evaluated}
}

\references{
Verspoor K, Cohn J, Mnizewski S, C J. A categorization approach to automated ontological function annotation. Protein Science. 2006;15:1544–1549.
}

\author{
Flavio E. Spetale <spetale@cifasis-conicet.gov.ar>
}

\examples{
data(CfData)

predGO <- matrix(runif(360, 0, 1),10,36, dimnames=list(rownames(
        CfData[["tableCfGO"]])[seq_len(10)], colnames(CfData[["tableCfGO"]])))

fMeasures(CfData[["tableCfGO"]][seq_len(10), ], predGO, cutoff = 0.5)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

fHierarchicalMeasures(CfData[["tableCfGO"]][seq_len(10), ], predGO, mygraphGO,
cutoff = 0.5)
}

\name{maxDistancegraphOnto}
\alias{maxDistancegraphOnto}

\title{
Maximum distance for a graph
}
\description{
Computes the maximum distance from any node to the root of the graph
}
\usage{
maxDistancegraphOnto(graphOnto)
}

\arguments{
    \item{graphOnto}{A graphNEL graph with `m' Ontology node labels.}
}

\details{
This function computes a distance matrix for a graph
}

\value{
Named numeric array containing the distance from any node to the root.
}

\seealso{
\code{\link[fgga]{fMeasure}}
}
\author{
Flavio E. Spetale <spetale@cifasis-conicet.gov.ar>
}

\examples{
data(CfData)

mygraphGO <- as(CfData[["graphCfGO"]], "graphNEL")

maxDistancegraphOnto(mygraphGO)
}

\name{tableTPG}
\alias{tableTPG}

\title{
Valid configurations for hidden variable nodes in a Forney Factor Graph
}
\description{
tableTPG provides valid configurations of hidden variable nodes at logical function nodes in a Forney Factor Graph under the True Path Graph (TPG) constraint.
}
\usage{
tableTPG(att)
}

\arguments{
    \item{att}{An integer indicating the number of cross-Ontology nodes involved}
}
\details{
Valid configurations of hidden variable nodes at logical function nodes enable messaging passing across the Forney Factor Graph. The TPG constraint is defined as: “If the child Ontology node describes the protein, then all its parent terms must also apply to that protein; and if a Ontology node does not describe a protein, then all its descendant Ontology nodes must not describe it”. The TPG constraint governs the structure of the Ontology-DAG and the inference process in the associated Forney Factor Graph.
}

\value{
A binary matrix with \emph{(n-1)^2+1} rows by \emph{n+1} columns where \emph{n = attr}
}
\references{
Tanoue J, Yoshikawa M, Uemura S (2012). The Gene Around GO viewer. Bioinformatics 18(12): 1705–1706.

Spetale FE, Tapia E, Krsticevic F, Roda F, Bulacio P (2016). A Factor Graph Approach to Automated GO Annotation. PLOS ONE 11(1): e0146986
}
\author{
Flavio E. Spetale, Pilar Bulacio and Javier Murillo <spetale@cifasis-conicet.gov.ar>
}

\examples{
tableTPG(3)
}

\keyword{TPG}

\name{CfData}
\alias{CfData}
\docType{data}
\title{
A set of characterized protein coding genes from the Cannis familiaris organism
annotated to a target GO subgraph considering both experimental and electronic
evidence.
}
\description{
The CfData dataset consists of a list containing the following:

\emph{$dxCf}: characterizations of \emph{6962} protein coding genes in terms of \emph{72} physico-chemical properties of their amino acid sequences. These sequences, obtained from the Uniprot database, are annotated to \emph{36} GO-terms of the GO Molecular Function (GO-MF) ontology subdomain.

\emph{$tableCfGO}: a set of \emph{6962} protein coding genes annotated to GO-MF target classes. Genes are identified by their \bold{Uniprot} ID mappings which are obtained with the org.Cf.eg.db annotation package set to work with both experimental and electronic evidence. Additionally, only those GO-MF terms with at least \emph{500} annotated genes were preserved.

\emph{$graphCfGO}: the target \bold{GO-MF} subgraph obtained with the org.Cf.eg.db annotation package set to work with the set of \bold{GO-MF} target classes.

\emph{$indexGO}: two arrays of Uniprot ID mappings defining the train-test partition of the set \emph{6962} protein coding genes annotated to \bold{GO-MF} terms.

\emph{$nodesGO}: labels of the GO-MF subgraph.

\emph{$varianceGOs}: a vector labeled with the variance of each \bold{GO-MF} term.
}
\usage{data("CfData")}
\format{
    A list with five named entries containg:
    \describe{
    \item{dxCf}{A matrix (6962 rows x 72 columns) containing the characterized proteins.}
    \item{graphCfGO}{An adjacency binary matrix (36 rows x 36 columns) corresponding to the GO-MF subgraph.}
    \item{indexGO}{A list with two named entries: indexTrain and indexTest each containing a numeric vector.}
    \item{tableCfGO}{A binary matrix (6962 rows x 36 columns) containing GOs associated with a protein.}
    \item{nodesGO}{A numerical vector containing the nodes of the GO-MF subgraph.}
    }
}


\source{
Uniprot Taxonomy: 9615

\url{https://www.uniprot.org/uniprot/?query=taxonomy:9615}

Package: org.Cf.eg.db - Version: 3.8.2

\url{https://bioconductor.org/packages/org.Cf.eg.db/}
}

\examples{
data(CfData)

## list objects included
ls(CfData)
# [1] "dxCf"  "graphCfGO" "indexGO"   "nodesGO"   "tableCfGO"

# Physico-chemical properties of each protein
head(CfData[["dxCf"]])

# GO-MF node labels, GO-terms, of each protein
head(CfData[["tableCfGO"]])

}
\keyword{datasets}
